#!/usr/bin/env python
# coding: utf-8

# In[1]:


# Evaluations described in the paper
# Studying Charge Migration Fragmentation of Sodiated Precursor Ions in Collision-Induced Fragmentation at the Library Scale
# 


# In[2]:


import pandas
import seaborn as sns
import matplotlib.pyplot as plt
import numpy as np
from collections import Counter
import math
import numpy as np
from brokenaxes import brokenaxes
import re


# In[3]:


##format lables in figures
def rotate_labels(ax, rotation=45):
    loc, labels = plt.xticks()
    ax.set_xticklabels(labels, rotation=rotation)
    plt.tight_layout()


# In[4]:


#create plots
def plotNumberPeakDistributions(dataArray, bins, colors, alpha=1, lw=1.5, xlim=(0,15), ylim=None, file=None, y_logscale=False, xlabel="number of peaks", ylabel="number of instances", hist=True, brokenaxisYLimits=None):
    if brokenaxisYLimits is None:
        fig, ax = plt.subplots(1,1, figsize=(6,4))
    else:
        fig = plt.figure(figsize=(6,4))
        bax = brokenaxes(ylims=brokenaxisYLimits, hspace=.05, fig=fig, despine=False)
        ax = bax
    
    for data,color in zip(dataArray, colors):
        y_data, label = data
        sns.distplot(y_data, ax=ax, label=label, bins=bins, norm_hist=False, kde=False, hist=hist, color=color, hist_kws={'alpha':alpha, 'lw':lw, 'edgecolor':color, 'fill':False, 'histtype':'step'})
        
    #xticks
    altbins = np.divide(range(0,1*6,1),5)  
    if brokenaxisYLimits is not None:        
        for currax in bax.axs:
            currax.set(xticks=np.add(altbins, 0), xticklabels=altbins)
    
        
    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    
    if brokenaxisYLimits is None:
        ax.set_xlim(xlim)
    else:
        for currax in bax.axs:
            currax.set_xlim(xlim)
    

    if ylim is not None:
        ax.set_ylim(ylim)
    if (y_logscale):
        plt.yscale('log')
    ax.legend()
    if (file is not None):
        plt.savefig(file, dpi=300)
    else:
        plt.plot()
        


# In[5]:


#set some default values

#mass deviation
deviation = '5ppm'
#file format of figure
file_format = "svg"

#colors
colors = ["blue", "green", "amber" , "purple", "grey"]
colors = sns.xkcd_palette(colors)


##settings only for relatvie number of peaks
hist=True
ylabel="number of compounds"


# In[ ]:





# In[6]:


#read peak statistics created from MS/MS spectra of sodiated and protonated precursor ions in NIST17
dfPeakStats = pandas.read_csv("paper-data/nist_saic_statistics.csv", sep="\t")


# In[7]:


#calculate bin sizes and min/max x of the data
bins_rel = list(np.divide(range(0,dfPeakStats['numPeaksHSpectrum'].max()*100,1),20)) 
bins_rel = list(np.divide(range(0,1*21,1),20))
bins_rel[-1]=bins_rel[-1]+0.00001  ##include the highest value. Else it might not be counted
xlim_rel=(0,1.0)

bins_abs = list(range(0,dfPeakStats['numPeaksHSpectrum'].max(),1))
bins_abs[-1]=bins_abs[-1]+0.00001  ##include the highest value. Else it might not be counted

xlim_abs=(0,15)


# In[8]:


len(dfPeakStats) 


# In[9]:


adduct="Na"
print('num. of fragments in [M+H]+ spectrum: %2.2f (%g)' % (dfPeakStats['numPeaksHSpectrum'].mean(), dfPeakStats['numPeaksHSpectrum'].median()))
print('num. of fragments in [M+Na]+ spectrum: %2.2f (%g)' % (dfPeakStats['numPeaksNaSpectrum'].mean(), dfPeakStats['numPeaksNaSpectrum'].median()))
print('num. of shared fragments: %2.2f (%g)' % (dfPeakStats['numPeaksMatching'].mean(), dfPeakStats['numPeaksMatching'].median()))
print('rel. num. of shared fragments: %2.2f%% (%2.2f%%)' % (dfPeakStats['ratioPeaksMatching'].mean()*100, dfPeakStats['ratioPeaksMatching'].median()*100))
print('num. of fragments with %s-H difference between spectra: %2.2f (%g)' % (adduct,dfPeakStats['numPeaksMatchingShift'].mean(), dfPeakStats['numPeaksMatchingShift'].median()))
print('rel. num. of %s-H difference between spectra: %2.2f%% (%2.2f%%)' % (adduct,dfPeakStats['ratioPeaksMatchingShift'].mean()*100, dfPeakStats['ratioPeaksMatchingShift'].median()*100))


# In[10]:


#number of different fragment ions in [M+H]+ and [M+Na]+ spectra
## a higher number of fragment ions might be due to the fact that H spectra have more collision energies
#
lw=1.5
ax = sns.distplot(dfPeakStats['numPeaksHSpectrum'], label='$\mathregular{[M+H]^{+}}$ spectra', kde=False, hist_kws={'alpha':1.0, 'lw':lw, 'fill':False, 'histtype':'step'})
ax = sns.distplot(dfPeakStats['numPeaksNaSpectrum'], label='$\mathregular{[M+Na]^{+}}$ spectra', kde=False, hist_kws={'alpha':1.0, 'lw':lw,'fill':False, 'histtype':'step'})
ax.set_xlabel("number of fragment ions")
ax.set_ylabel("number of compounds")
plt.legend()
# plt.savefig('figures/numberOfPeaksInSpectrum.png', dpi=300)
plt.savefig('figures/numberOfPeaksInSpectrum.%s'%file_format)


# In[11]:


dataArray = [
             [dfPeakStats['ratioPeaksMatching'], 'shared fragment ions between spectra'],
             [dfPeakStats['ratioPeaksMatchingShift'], '$\Delta$(Na-H) between spectra'],
             [dfPeakStats['ratioPeaksMatchingShift_within_Na_spectrum'], '$\Delta$(Na-H) in $\mathregular{[M+Na]^{+}}$ spectrum'],
            [dfPeakStats['ratioPeaksMatchingShift_within_H_spectrum'], '$\Delta$(Na-H) in $\mathregular{[M+H]^{+}}$ spectrum'],
            ]

ylim=(0,8)
file = 'figures/countPeaks_rel_%s_na_h_shift.%s'%(deviation, file_format)
ylim=None

brokenaxisYLimits=((0,750), (1350,1450), (1750, 1820))
plotNumberPeakDistributions(dataArray, bins=bins_rel, colors=colors, file=file, xlim=xlim_rel, ylim=ylim, y_logscale=False, xlabel="relative number of fragment ions", ylabel=ylabel, hist=hist, brokenaxisYLimits=brokenaxisYLimits)


# In[12]:


#number of Na-H matches in [M+H]+ spectrum
moreThanZero = len(dfPeakStats[dfPeakStats['ratioPeaksMatchingShift_within_H_spectrum']!=0.0])/len(dfPeakStats)
moreThanOnePercOfRelNumPeaks = len(dfPeakStats[dfPeakStats['ratioPeaksMatchingShift_within_H_spectrum']>0.01])/len(dfPeakStats)
print("at least some match in %.2f%%"%(100.0*moreThanZero))
print("more than 1%% of rel. number of fragments with match in %.2f%%"%(100.0*moreThanOnePercOfRelNumPeaks))


# In[13]:


dataArray = [
              [dfPeakStats['intensityMatching'], 'shared fragment ions between spectra'],
             [dfPeakStats['intensityMatchingShift'], '$\Delta$(Na-H) between spectra'],
             [dfPeakStats['intensityMatchingShift_within_Na_spectrum'], '$\Delta$(Na-H) in $\mathregular{[M+Na]^{+}}$ spectrum'],
            [dfPeakStats['intensityMatchingShift_within_H_spectrum'], '$\Delta$(Na-H) in $\mathregular{[M+H]^{+}}$ spectrum']    
            ]

ylim=None
brokenaxisYLimits=((0,520), (850,930), (1450, 1520), (1750, 1820))
file = 'figures/intensity_rel_%s_na_h_shift.%s'%(deviation, file_format)
plotNumberPeakDistributions(dataArray, bins=bins_rel, colors=colors, file=file, xlim=xlim_rel, ylim=ylim, y_logscale=False, xlabel="relative intensity", hist=hist, ylabel=ylabel, brokenaxisYLimits=brokenaxisYLimits)


# In[ ]:





# In[ ]:





# In[15]:


fig, ax = plt.subplots(1,1, figsize=(5,5))

xcol='ratioPeaksMatching'
ycol='ratioPeaksMatchingShift'

maxi=.51

selectiondf = dfPeakStats.groupby([xcol, ycol]).count().rename(columns={'numSpectraH':'frequency'})
selectiondf = selectiondf.reset_index()
selectiondf = selectiondf[(selectiondf[xcol]!=0) | (selectiondf[ycol]!=0)]

cmap = sns.cubehelix_palette(dark=.1, light=.8, as_cmap=True)
sns.scatterplot(x=xcol, y=ycol, data=selectiondf, ax=ax, size='frequency',  hue='frequency', palette=cmap)

ax.plot((0,1), (1,0), linestyle='--', color='lightgray')

ax.set_ylabel('ratio of $\mathregular{[M]^{+}}$ fragment ions')
ax.set_xlabel('ratio of $\mathregular{[M+Na]^{+}}$ fragment ions')

plt.tight_layout()
plt.savefig('figures/ratio_protonated_vs_sodiated_fragments.%s'%file_format)
# plt.savefig('ratio_protonated_vs_sodiated_fragments.png', dpi=400)


# In[16]:


fig, ax = plt.subplots(1,1, figsize=(5,5))

xcol='ratioAdductSwitchesWithinNaSpectrum'
ycol='ratioAdductSwitchesWithinNaSpectrumMatchingASharedPeakInHSpectrum'

maxi=1.00
maxi=.50

selectiondf = dfPeakStats.groupby([xcol, ycol]).count().rename(columns={'numSpectraH':'frequency'})
selectiondf = selectiondf.reset_index()
selectiondf = selectiondf[(selectiondf[xcol]!=0) | (selectiondf[ycol]!=0)]

vmin, vmax = 1,100
norm=plt.Normalize(vmin=vmin, vmax=vmax)
norm_max = selectiondf['frequency'].max()
cmap = sns.cubehelix_palette(dark=.1, light=.8, as_cmap=True)
sns.scatterplot(x=xcol, y=ycol, data=selectiondf, ax=ax, size='frequency',  hue='frequency', palette=cmap)

ax.plot((0,maxi), (0, maxi), linestyle='--', color='lightgray')

ax.set_xlabel('ratio of $\mathregular{[M]^{+}}$ fragment ions annotated by\n$\Delta$(Na-H) within sodiated ion spectrum')
ax.set_ylabel('ratio of fragment ions annotated as $\mathregular{[M]^{+}}$ and\nhaving shared fragment in protonated ion spectrum')

plt.tight_layout()
plt.savefig('figures/ratio_fragments_matching_both_within_and_between.%s'%file_format)
# plt.savefig('ratio_fragments_matching_both_within_and_between.png', dpi=400)


# In[17]:


###frequent mz differences####
frequentDifferencesDf = pandas.read_csv("paper-data/frequent-mass-differnces-sodium-spectra.tsv", sep="\t")
frequentDifferencesDf.head(15)


# In[18]:


######################################################################################################
####Functional groups evalution#######################################################################
######################################################################################################


# In[19]:


dfGroupStats = pandas.read_csv("paper-data/functional-group-statistics-individual-groups.tsv",sep="\t")
dfGroupStats


# In[20]:


dfGroupStats2 = pandas.read_csv("paper-data/functional-group-statistics-combined-groups.tsv",sep="\t")
dfGroupStats2


# In[21]:


dfGroupAndPeakRatios = pandas.read_csv("paper-data/functional-group-statistics-combined-groups-per-instance.tsv",sep="\t")


# In[22]:


#format numbers in groups 

formatted_groups = []
for g in dfGroupAndPeakRatios['group']:
    last = " "+g.split(" ")[-1]
    first = [re.sub(r'([23456789])', r'$_\1$', s) for s in g.split(" ")[0:-1]]
    first = " ".join(first)
    formatted_groups.append(first+last)

dfGroupAndPeakRatios['group'] = formatted_groups


# In[23]:


# relative number of fragments shared between [M+H]+ and [M+Na]+ spectrum 
# depending on the existence of functional groups

ycategory='ratioPeaksMatching'

plt.style.use("default")

fig, ax = plt.subplots(1,1, figsize = (8,5));
hline_y = dfGroupAndPeakRatios[dfGroupAndPeakRatios['group']=='all data (1149)'][ycategory].median()
print(hline_y)
ax.hlines(hline_y, -0.5, 21.5, 'red', linestyles='-', zorder=0)
ax = sns.boxplot(x='group', y=ycategory, data=dfGroupAndPeakRatios, ax=ax, notch=True, zorder=2, color='black')
plt.setp(ax.artists, facecolor='w')
rotate_labels(ax, 90)
plt.ylabel("rel. number of\nshared fragments")
plt.tight_layout()
# plt.savefig("functional_groups_boxplot_peaks_ratios.png", dpi=400)
plt.savefig("figures/functional_groups_boxplot_peaks_ratios.%s"%file_format)


# In[24]:


######################################################################################################
## peak intensity ratios within sodium spectra #######################################################
######################################################################################################


# In[25]:


dfIntensityShiftInSameSpece = pandas.read_csv('paper-data/peak_statistics_within_sodium_spectrum.tsv', sep='\t')


# In[26]:


dfIntensityShiftInSameSpece['ratio'] = np.divide(dfIntensityShiftInSameSpece['H_rel_intensity'], dfIntensityShiftInSameSpece['Na_rel_intensity'])
dfIntensityShiftInSameSpece['ratioLog'] = np.log2(np.divide(dfIntensityShiftInSameSpece['H_rel_intensity'], dfIntensityShiftInSameSpece['Na_rel_intensity']))


print('median ratio Na-H pair', np.median(dfIntensityShiftInSameSpece['ratio']))
print('median H intensity of Na-H pair', np.median(dfIntensityShiftInSameSpece['H_rel_intensity'])*100)
print('median sodium intensity Na-H pair', np.median(dfIntensityShiftInSameSpece['Na_rel_intensity'])*100)
    
print('mean log ratio Na-H pair ', np.mean(dfIntensityShiftInSameSpece['ratioLog']))
print('mean H intensity Na-H pair', np.mean(dfIntensityShiftInSameSpece['H_rel_intensity'])*100)
print('mean sodium intensity Na-H pair', np.mean(dfIntensityShiftInSameSpece['Na_rel_intensity'])*100)


# In[27]:


print('number of pairs in same spec %d'%len(dfIntensityShiftInSameSpece))


# In[28]:


# plot relative intensities of [M]+ and [M+Na]+ fragments in [M+Na]+ spectrum

bins = np.arange(0,1.01,0.05)

brokenaxisYLimits=((0, 180), (830,970))
fig = plt.figure(figsize=(7,5))
bax = brokenaxes(ylims=brokenaxisYLimits, hspace=.05, fig=fig, despine=False)
ax = bax


norm=False
kde=False
hist=True
alpha=1
lw=1.5
current_palette = sns.color_palette()

y_data = dfIntensityShiftInSameSpece['H_rel_intensity']
label="$\mathregular{[M]^{+}}$ intensity"
color= current_palette[0]
sns.distplot(y_data, ax=ax, label=label, bins=bins,norm_hist=norm, kde=kde, hist=hist, color=color, hist_kws={'alpha':alpha, 'lw':lw, 'edgecolor':color, 'fill':False, 'histtype':'step'})

y_data = dfIntensityShiftInSameSpece['Na_rel_intensity']
label="$\mathregular{[M+Na]^{+}}$ intensity"
color= current_palette[1]
sns.distplot(y_data, ax=ax, label=label, bins=bins,norm_hist=norm, kde=kde, hist=hist, color=color, hist_kws={'alpha':alpha, 'lw':lw, 'edgecolor':color, 'fill':False, 'histtype':'step'})


ax.set_xlabel('relative intensity')
ax.set_ylabel('number of fragment ions')

ax.legend()

for currax in bax.axs:
    currax.set_xlim((0,1))

# plt.savefig('intensity_distributiuon_in_Na_matched_within.png', dpi=400)
plt.savefig('figures/intensity_distributiuon_in_Na_matched_within.%s'%file_format)


# In[29]:


######################################################################################################
###evalutation based on ClassyFire compound classes ##################################################
######################################################################################################

# here we compare the number of [M+Na]+ spectra with at least 20% protonated fragments against all [M+Na]+ spectra


# In[30]:


dfCombinedClassyfireInfo = pandas.read_csv("paper-data/classyfire-statistics.tsv", sep="\t")


# In[31]:


dfCombinedClassyfireInfo


# In[32]:


# Initialize the matplotlib figure
f, ax = plt.subplots(figsize=(8, 6))

bardata = dfCombinedClassyfireInfo.sort_values("number of instances (all)", ascending=False)

for x in [50,100,150,200,250,300]:
    ax.vlines(x, -0.4, 11.5, 'lightgrey', linestyles='--')


classyFireHierarchy = dfCombinedClassyfireInfo.columns[0]
sns.set_color_codes("pastel")
sns.barplot(x="number of instances (all)", y=classyFireHierarchy, data=bardata,
            label="total", color="b", ax=ax, zorder=2)

sns.set_color_codes("muted")
sns.barplot(x="number of instances (frequent protonation)", y=classyFireHierarchy, data=bardata,
            label="20% protonated fragment ions ", color="b", ax=ax, zorder=2)


# Add a legend and informative axis label
ax.legend(ncol=2, loc="lower right", frameon=True)

ax.set(ylabel="",
       xlabel="number of compounds")
sns.despine(left=True, bottom=True)
plt.tight_layout()
# plt.savefig("classyfire_barchart.png", dpi=300)
plt.savefig('figures/classyfire_barchart.%s'%file_format)


# In[ ]:




